Package: Informeasure
Type: Package
Title: R implementation of information measures
Version: 1.22.0
Authors@R: c(person(given = "Chu", family = "Pan", role = c("aut", "cre"), email = "chu.pan@hnu.edu.cn"))
Description: This package consolidates a comprehensive set of information measurements, encompassing mutual information, conditional mutual information, interaction information, partial information decomposition, and part mutual information.
License: Artistic-2.0
Depends: R (>= 4.0)
Encoding: UTF-8
RoxygenNote: 7.3.1
biocViews: GeneExpression, NetworkInference, Network, Software
Imports: entropy
Suggests: knitr, BiocStyle, rmarkdown, testthat (>= 3.0.0),
        SummarizedExperiment
VignetteBuilder: knitr
URL: https://github.com/chupan1218/Informeasure
BugReports: https://github.com/chupan1218/Informeasure/issues
Config/testthat/edition: 3
git_url: https://git.bioconductor.org/packages/Informeasure
git_branch: RELEASE_3_23
git_last_commit: f0c75ce
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 22:44:56 UTC; biocbuild
Author: Chu Pan [aut, cre]
Maintainer: Chu Pan <chu.pan@hnu.edu.cn>
